import argparse

from sqlalchemy import create_engine, text
from sqlalchemy.orm import Session, configure_mappers
from postgresql_audit import versioning_manager

from ..models import Base
from .utils import get_db_dsn


def main():

    parser = argparse.ArgumentParser(description="Initialize UMnetdb database")
    parser.add_argument(
        "--env-file",
        help="Path to environment file - you can also set this in your eviroment as UMNETDB_ENV.",
    )
    args = parser.parse_args()

    db_url = get_db_dsn(args.env_file)
    
    engine = create_engine(db_url)
    print(f"connecting to {db_url}")
    engine.connect()
    print("done")
    with Session(engine) as session:

        # extension needed for postresql-audit; https://github.com/kvesteri/postgresql-audit/issues/44
        session.execute(text("CREATE EXTENSION btree_gist;"))
        session.commit()

        configure_mappers()

        # need to create versioning manager tables before creating all other tables
        # if not session.execute(text("select table_name from information_schema.tables where table_name='activity'")):
        versioning_manager.transaction_cls.__table__.create(engine)
        versioning_manager.activity_cls.__table__.create(engine)

        Base.metadata.create_all(engine)
        session.commit()


if __name__ == "__main__":
    main()



import argparse

from sqlalchemy import create_engine, select, or_
from sqlalchemy.orm import Session

from postgresql_audit import versioning_manager

from ..models import Base, Versioned
from .utils import get_db_dsn, COLUMNS_BY_TABLE


def main():

    parser = argparse.ArgumentParser(
        description="Query the activity table for changes to the database"
    )
    parser.add_argument(
        "--env-file",
        help="Path to environment file - you can also set this in your eviroment as UMNETDB_ENV.",
    )
    # adding subparsers for each table column
    subparsers = parser.add_subparsers(dest="table")
    for mapper in Base.registry.mappers:

        # skip non-versioned tables
        if not issubclass(mapper.class_, Versioned):
            continue

        t = mapper.local_table

        table_parser = subparsers.add_parser(
            t.name, help=f"Search for activity around the {t.name} table"
        )
        for col in t.c:
            table_parser.add_argument(
                f"--{col.name}", help=f"Search by {col.name} column"
            )
        table_parser.add_argument(
            "--columns",
            nargs="*",
            help="Restrict display to specific columns",
            choices=[c.name for c in t.c],
        )

    args = parser.parse_args()

    engine = create_engine(get_db_dsn(args.env_file))

    activity = versioning_manager.activity_cls

    # querying the activity table
    query = select(activity.issued_at, activity.verb, activity.changed_data).where(
        activity.table_name == args.table
    )

    for col_name in COLUMNS_BY_TABLE[args.table]:
        col_query = getattr(args, col_name)
        if col_query:
            query = query.filter(
                or_(
                    activity.changed_data[col_name].astext == col_query,
                    activity.old_data[col_name].astext == col_query,
                )
            )

    # execute query and print the result
    with Session(engine) as s:
        result = s.execute(query)

        # need to unroll
        for r in result:
            print(f"{r.issued_at}: {r.verb} {r.changed_data}")

        # need to convert to dict and un-nest
        # data = [r[0] for r in list(result)]

        # need to un-nest 'changed data'

        # print(data[0].as_dict())
        # print(generate_cli_table(data[0].keys(), [r.values() for r in data]))


if __name__ == "__main__":
    main()



import argparse
from typing import Union

from sqlalchemy import create_engine, select, or_, cast, String, text, Column
from sqlalchemy.orm import Session

from ..models import Base
from .utils import (
    is_ip_or_prefix,
    is_prefix,
    get_db_dsn,
    COLUMNS_BY_TABLE,
    generate_cli_table,
)


inet_ops = {
    "le": "<<=",
    "ge": ">>=",
    "eq": "=",
}


def generate_where_statement(
    col: Column, search: str, arg_inet_op: Union[str, None]
) -> str:
    """
    Generates where statement based on the datatype of the column we're searching
    or if there's a wildcard (%) in the search string
    """

    # if this is an inet type column do an IP based search
    if str(col.type) == "INET" and is_ip_or_prefix(search):

        # setting the inet op. If the user provided what they wanted to use
        # use, that.
        if arg_inet_op:
            inet_op = inet_ops[arg_inet_op]

        # if the search is a prefix-based search, assume we're looking for
        # hosts within a network (find longer-prefix matches)
        elif is_prefix(search):
            inet_op = ">>="

        # if the search is a host-based search, assume we're looking for
        # the network within the host resides (shorter-prefix match)
        else:
            inet_op = "<<="

        where = text(f"'{search}' {inet_op} {col}")

    # if search string has a '%' in it do a 'like' search
    elif "%" in search:
        where = cast(col, String).like(search)

    # default is exact column search
    else:
        where = col == search

    return where


supbarser_epilog = """
Note that a global search cannot be used when also specifying a column to search.
When you do specify columns, the search will 'and' these logically.

Global search looks for your string in all columns and includes substrings.
Column searches are exact but if you use "%" in your query it'll use a 'like' search to capture
wildcard matches
"""


def main():

    parser = argparse.ArgumentParser(description="Query a umnet_db table")
    parser.add_argument(
        "--env-file",
        help="Path to environment file - you can also set this in your eviroment as UMNETDB_ENV.",
    )

    # adding subparsers for each table column
    subparsers = parser.add_subparsers(dest="table")
    for t, cols in COLUMNS_BY_TABLE.items():

        # skip activity/diff tables
        if t in ["transaction", "activity"]:
            continue

        table_parser = subparsers.add_parser(
            t, help=f"Search the {t} table", epilog=supbarser_epilog
        )
        table_parser.add_argument(
            "global_search", nargs="?", help="Global search of the table"
        )
        for col_name in cols:
            table_parser.add_argument(
                f"--{col_name}", help=f"Search by {col_name} column"
            )
        table_parser.add_argument(
            "--columns",
            nargs="*",
            help="Restrict display to specific columns",
            choices=cols,
        )
        table_parser.add_argument(
            "--inet-op",
            choices=inet_ops,
            help="for IP searches, match on 'le' (contains), 'ge' (contained by), or 'eq' exact match",
        )
        table_parser.add_argument(
            "--sort-by", help="Sort by a particular column (ascending)", choices=cols
        )
        table_parser.add_argument(
            "--descending", help="Reverse sort (descending)", action="store_true"
        )

    args = parser.parse_args()

    if not args.table:
        print(
            "Please specify which table to query. Run this command with '-h' for details."
        )
        exit(1)

    # we either want to search for a single term across all columns,
    # or for specific values in specific columns, not both.
    if args.global_search and len(
        [
            a
            for a in args.__dict__
            if a in COLUMNS_BY_TABLE[args.table] and getattr(args, a)
        ]
    ):
        print(
            "ERROR: Global search combined with column-specific searches is not supported"
        )
        exit(1)

    # Sort parameter. If none provided sort by device - for the device table this is
    # the 'name' column, for every other table it's 'device'
    if args.sort_by:
        sort_by = args.sort_by
    elif args.table == "device":
        sort_by = "name"
    else:
        sort_by = "device"

    engine = create_engine(get_db_dsn(args.env_file))

    table = Base.metadata.tables[args.table]

    if args.columns:
        query = select(table.c[tuple(args.columns)])
    else:
        query = select(table)

    # Adding 'where' statements based on columns or global search queries
    if args.global_search:
        query = query.where(
            or_(
                *[
                    generate_where_statement(c, args.global_search, args.inet_op)
                    for c in table._columns
                ]
            )
        )
    # If user inputted columns to query, we want to do do an 'and' query
    else:
        for col_name in COLUMNS_BY_TABLE[args.table]:
            col_search = getattr(args, col_name)
            if col_search:
                query = query.where(
                    generate_where_statement(
                        table._columns[col_name], col_search, args.inet_op
                    )
                )

    # add on our 'sort by'
    if args.descending:
        query = query.order_by(table._columns[sort_by].desc())
    else:
        query = query.order_by(table._columns[sort_by])

    # execute query and print the result
    with Session(engine) as s:
        result = s.execute(query)
        print(generate_cli_table(result.keys(), list(result)))


if __name__ == "__main__":
    main()



class UMNetDBError(Exception):
    """
    Generic umnetdb error
    """

    pass


class UMnetDBModelError(Exception):
    """
    Error related to data models
    """

    pass


class UMnetDBLookupError(Exception):
    """ "
    Error related to queries
    """

    def __init__(self, query, table):
        super().__init__(f"Could not find {query} in {table.__name__} table")



import argparse
from sqlalchemy import create_engine
from sqlalchemy.orm import Session
import re

from ..models import Base
from .utils import get_db_dsn


def main():

    parser = argparse.ArgumentParser(description="Clear our drop a table or all tables")
    parser.add_argument("table", choices=list(Base.metadata.tables).append("all"))
    parser.add_argument(
        "--env-file",
        help="Path to environment file - you can also set this in your eviroment as UMNETDB_ENV.",
    )
    parser.add_argument(
        "--drop",
        action="store_true",
        help="Actually drop the table instead of just deleting the contents",
    )
    args = parser.parse_args()

    if args.table == "all":
        are_you_sure = input(
            "Are you sure you want to delete ALL tables? Type y or yes to confirm: "
        )
    else:
        are_you_sure = input(
            f"Are you sure you want to delete {args.table}? Type y or yes to confirm: "
        )
    if not (re.match("(y|yes)$", are_you_sure, re.IGNORECASE)):
        print("ok, never mind")
        exit(1)

    engine = create_engine(get_db_dsn(args.env_file))

    with Session(engine) as session:
        if args.table == "all" and args.drop:
            Base.metadata.drop_all(engine)

        elif args.table == "all":
            for table in Base.metadata.tables.values():
                session.execute(table.delete())

        elif args.drop:
            Base.metadata.tables[args.table].drop(engine)
        else:
            session.execute(Base.metadata.tables[args.table].delete())

        session.commit()


if __name__ == "__main__":
    main()



from typing import List
from decouple import Config, RepositoryEnv, UndefinedValueError
from os import getenv
from texttable import Texttable
from netaddr import IPNetwork, EUI
from netaddr.core import AddrFormatError

from ..errors import UMNetDBError
from ..models import Base

MIN_COL_WIDTH = 5
MAX_COL_WIDTH = 50
MAX_TABLE_WIDTH = 0

COLUMNS_BY_TABLE = {k: [c.name for c in v.c] for k, v in Base.metadata.tables.items()}


def get_db_dsn(env_file: str):
    """
    Generates DSN based on provided environment file. If provided file is none, checks
    for path to file in environment variable UMNETDB_ENV
    """
    env_file = env_file if env_file else getenv("UMNETDB_ENV")
    if not env_file:
        raise UMNetDBError("No config file provided, and no UMNETDB_ENV env set!")

    try:
        env = Config(RepositoryEnv(env_file))
    except FileNotFoundError:
        raise UMNetDBError(f"Cannot locate umnet_db environment file {env_file}")

    e = {}
    for var in ["DB_USER", "DB_PASS", "DB_HOST", "DB_PORT", "DB_NAME"]:
        try:
            e[var] = env.get(var)
        except UndefinedValueError:
            raise UMNetDBError(f"Need to set {var} in {env_file}")

    return f"postgresql+psycopg://{e['DB_USER']}:{e['DB_PASS']}@{e['DB_HOST']}:{e['DB_PORT']}/{e['DB_NAME']}"


def generate_cli_table(header: list, rows: List[list]) -> str:
    """
    Takes in a header and rows and draws a texttable.
    Column widths are dynamically set based on the longest entry in each column,
    limited by MAX_COL_WIDTH
    """

    table = Texttable()
    table.set_deco(Texttable.HEADER)

    # starting with minimum column widths and creating a header
    num_cols = len(header)
    table.set_cols_width([MIN_COL_WIDTH] * num_cols)
    table.add_row(header)
    table.set_max_width(MAX_TABLE_WIDTH)

    table.set_cols_width([len(c) for c in header])
    table.set_cols_dtype(["t" for c in header])

    for row in rows:

        row_widths = [len(str(c)) for c in row]
        for t_col_width, r_col_width, idx in zip(
            table._width, row_widths, range(0, num_cols)
        ):
            if r_col_width > t_col_width and r_col_width <= MAX_COL_WIDTH:
                table._width[idx] = r_col_width

        table.add_row(row)

    return table.draw()


def is_ip_or_prefix(ip: str) -> bool:
    """
    Returns whether a string is an ip (10.233.0.10) or
    IP + prefix (10.233.0.10/24). IPv6 works as well.
    """
    try:
        IPNetwork(ip)
    except AddrFormatError:
        return False
    return True


def is_prefix(ip: str) -> bool:
    """
    Returns whether a string is a prefix or not.
    Host routes don't count!
    """
    try:
        ip_obj = IPNetwork(ip)
    except AddrFormatError:
        return False

    # host routes don't count as prefixes!
    if (ip_obj.prefixlen == 32 and ip_obj.version == 4) or (
        ip_obj.prefixlen == 128 and ip_obj.version == 6
    ):
        return False

    return True


def is_mac(mac: str) -> bool:
    """
    Returns whether this is a valid mac address or not
    """
    try:
        EUI(mac)
    except AddrFormatError:
        return False
    return True
